/*********************************
 * This file is part of the "Interactive Fan Chart Demo"
 * by Geoffrey M. Draper.
 * 
 * Geoffrey M. Draper, author of this work, hereby grants
 * everyone permission to use and distribute this code,
 * with or without modifications for any purpose,
 * commercial or otherwise. 
 * 
 * It is requested, although not required, that derivative
 * works include an acknowledgment somewhere in the
 * derivative work's credits.
 *********************************/

package fanchart;

import java.util.ArrayList;
import java.util.List;

public class Person implements Comparable<Person> {
	private String name;
	private String birthdate;//not used yet
	private Person mother;
	private Person father;
	private List<Person> children;
	private boolean changed;
	private int generation;
	private int ID;
	private static int GENERATION_NOT_INITIALIZED = Integer.MAX_VALUE;
	
	public Person(String name, String birth) {
		this.name = name;
		birthdate = birth;
		mother = null;
		father = null;
		children = new ArrayList<Person>();
		changed = false;
		generation = GENERATION_NOT_INITIALIZED;
		recomputeID();
	}
	
	public Person(Person p) {
		this.name = new String(p.name);
		this.birthdate = new String(p.birthdate);
		mother = p.mother;
		father = p.father;
		children = new ArrayList<Person>(p.children);
		changed = false;
		generation = p.generation;
		this.ID = p.ID;
	}
	
	public Person(String name) {
		this(name, "");
	}

	public Person(String name, Person f, Person m) {
		this(name, "");
		setParents(f,m);
	}
	
	public Person setName(String name) {
		this.name = name;
		recomputeID();
		return this;
	}
	
	public Person setBirthdate(String bd) {
		birthdate = bd;
		recomputeID();
		return this;
	}
	
	public String getBirthdate() {
		return birthdate;
	}
	
	public Person setMother(Person m) {
		mother = m;
		mother.addChild(this);
		recomputeID();
		return this;
	}
	
	public boolean addChild(Person p) {
		return children.add(p);
	}
	
	public String getName() {
		return name;
	}
	
	public Person setFather(Person f) {
		father = f;
		father.addChild(this);
		recomputeID();
		return this;
	}
	
	public void setParents(Person f, Person m) {
		setMother(m);
		setFather(f);
		//no need to recomputeID since the above methods do it.
	}
	
	public int getID() {
		return ID;
	}
	
	private void recomputeID() {
		String kidsnames = "";
		String motherName = ((mother == null) ? "" : mother.getName());
		String fatherName = ((father == null) ? "" : father.getName());
		for (Person p : children) {
			kidsnames += p.getName();
		}
		ID = (name + kidsnames + fatherName + motherName + birthdate).hashCode();
	}

	public Person getMother() {
		return mother;
	}

	public Person getFather() {
		return father;
	}
	
	public Person getChild() {
		return children.get(0);
	}
	
	public List<Person> getChildren() {
		return children;
	}
	
	public Person getChild(int i) {
		return children.get(i);
	}
	
	public boolean hasChild() {
		return (!(children.isEmpty()));
	}
	
	@Override
	public boolean equals(Object other) {
		boolean result = false;
		if (other instanceof Person) {
			result = (ID == ((Person)other).ID);
		}
		return result;
	}

	public void setChanged(boolean b) {
		changed = b;
	}
	
	public boolean isChanged() {
		return changed;
	}

	public boolean hasMultipleChildren() {
		return (children.size() > 1);
	}

	public int getNumChildren() {
		return children.size();
	}
	
	//NOTE: this method is to be called ONLY at program
	//startup time, and ONLY on the initial root node.
	public void setGeneration(int g) {
		if (generation == GENERATION_NOT_INITIALIZED) {
			generation = g;
			if (father != null) father.setGeneration(generation+1);
			if (mother != null) mother.setGeneration(generation+1);
			for (Person c : children) {
				c.setGeneration(generation-1);
			}
		}
	}
	
	//NOTE: this method is to be called ONLY at program
	//startup time, and ONLY on the initial root node.
	public void addToList(List<Person> list) {
		if (list.contains(this)) return;
		list.add(this);
		if (father != null) father.addToList(list);
		if (mother != null) mother.addToList(list);
		for (Person c : children) {
			c.addToList(list);
		}
	}

	
	public int getGeneration() {
		return generation;
	}

	public int compareTo(Person o) {
		return name.compareTo(o.name);
	}
		
}
